import { DynamicJsonfield } from "./components/dynamicComponents/DynamicJsonfield";
import { DynamicBooleanField } from "./components/dynamicComponents/DynamicBooleanField";
import { DynamicDateField } from "./components/dynamicComponents/DynamicDateField";
import { DynamicPredefinedArrayField } from "./components/dynamicComponents/DynamicPredefinedArrayField";
import { DynamicTextfield } from "./components/dynamicComponents/DynamicTextfield";
import React from "react";
import { FluentPropertyLabel } from "./components/FluentPropertyLabel";
import { FluentFormShimmer } from "./components/list/components/FluentFormShimmer";
import FluentList from "./components/list/FluentList";
import { buildPropertyRenderInfo, getValidationMarkForProperty, propertyTypes } from "@wiberg/formbuilder";
export const createFluentBuilder = (labelRender) => {
    const id = "internal_fluentbuilder";
    const DefaultLabelRender = FluentPropertyLabel;
    const BuilderLabelRender = labelRender !== null && labelRender !== void 0 ? labelRender : DefaultLabelRender;
    const listComponent = () => FluentList;
    const loadingSpinnerComponent = () => FluentFormShimmer;
    const build = (renderProps, property, schema) => {
        let { item } = renderProps;
        if (item === null)
            throw Error("item is null");
        if (schema == null)
            throw Error("schema is null");
        let info = buildPropertyRenderInfo(renderProps, schema, property);
        let validationMark = getValidationMarkForProperty(renderProps, property);
        const WrapInLabel = (element) => {
            return (React.createElement("div", { className: "formbuilder-property", key: info.key },
                React.createElement(BuilderLabelRender, { key: `${info.key}-labelcontainer`, propertySchema: schema, hideLabel: info.props.options.hideLabel, parentKey: info.key, validationMark: validationMark }),
                element));
        };
        const propertyType = propertyTypes;
        switch (schema.propertyType) {
            case propertyType.string: return { found: true, element: WrapInLabel(React.createElement(DynamicTextfield, Object.assign({}, schema, info.props))) };
            case propertyType.number: return { found: true, element: WrapInLabel(React.createElement(DynamicTextfield, Object.assign({}, schema, info.props))) };
            case propertyType.boolean: return { found: true, element: WrapInLabel(React.createElement(DynamicBooleanField, Object.assign({}, schema, info.props))) };
            case propertyType.date: return { found: true, element: WrapInLabel(React.createElement(DynamicDateField, Object.assign({}, schema, info.props))) };
            case propertyType.json: return { found: true, element: WrapInLabel(React.createElement(DynamicJsonfield, Object.assign({}, schema, info.props))) };
            case propertyType.predefinedArray: return { found: true, element: WrapInLabel(React.createElement(DynamicPredefinedArrayField, Object.assign({}, schema, info.props))) };
            default: return { found: false, element: undefined };
        }
    };
    return { id, build, loadingComponent: loadingSpinnerComponent, listComponent };
};
//# sourceMappingURL=FluentBuilder.js.map