var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { Dialog, DialogFooter, PrimaryButton, DefaultButton, CommandBar, ShimmeredDetailsList, SelectionMode, DetailsListLayoutMode, ConstrainMode, Selection, Icon, Stack, Spinner } from "@fluentui/react";
import { useState, useEffect, useRef, useCallback } from "react";
import { FluentDialog } from "./FluentDialog";
import React from "react";
import { useStateRef, formListHelper, ValidationResult, lang, id } from "@wiberg/formbuilder";
import { Searcher } from "../..";
const FluentList = (props) => {
    var _a, _b, _c, _d, _e, _f, _g, _h, _j, _k, _l, _m, _o, _p;
    const editorFormRef = useRef();
    const [items, setItems] = useState(props.items);
    const [filteredItems, setFilteredItems] = useState(props.filteredItems);
    const [showEditor, setShowEditor] = useState(false);
    const [columns, setColumns, columnRef] = useStateRef();
    const [showValidationOverrideConfirm, setShowValidationOverrideConfirm] = useState(false);
    const [enableShimmer, setEnableShimmer] = useState(false);
    const [newItemMode, setNewItemMode] = useState(false);
    const [menuItems, setMenuItems] = useState(undefined);
    const [validationFailed, setValidationFailed] = useState(false);
    const [formLoading, setFormLoading] = useState(false);
    const [selection] = useState(new Selection({
        selectionMode: props.listProps.config.multiSelect ? SelectionMode.multiple : SelectionMode.single,
        getKey: (item, index) => props.listProps.config.itemIdentifier(item),
        onSelectionChanged: () => {
            var _a;
            props.updateSelectedItems((_a = selection.getSelection()) !== null && _a !== void 0 ? _a : []);
        }
    }));
    useEffect(() => {
        let cols = buildColumns();
        setColumns(cols);
    }, [props.columns]);
    useEffect(() => {
        setEnableShimmer(props.shimmer);
    }, [props.shimmer]);
    useEffect(() => {
        var _a;
        if ((_a = props.items) === null || _a === void 0 ? void 0 : _a.length)
            setEnableShimmer(false);
        setItems(props.items);
    }, [props.items]);
    useEffect(() => {
        setFilteredItems(props.filteredItems);
    }, [props.filteredItems]);
    const buildMenuItem = useCallback((item) => {
        var _a;
        return {
            key: item.title,
            text: item.title,
            onClick: () => item.action(props.selectedItems),
            disabled: props.readOnly || formListHelper.isMenuItemDisabled(item.selectionMode, (_a = props.selectedItems) === null || _a === void 0 ? void 0 : _a.length),
            iconProps: { iconName: item.iconName },
            subMenuProps: item.subMenuItems == null
                ? undefined
                : { items: item.subMenuItems.map(buildMenuItem) }
        };
    }, [items, props.selectedItems, props.readOnly]);
    useEffect(() => {
        var _a, _b;
        if (((_a = props.listProps.menuConfig) === null || _a === void 0 ? void 0 : _a.actions) == null)
            return;
        let actions = (_b = props.listProps.menuConfig) === null || _b === void 0 ? void 0 : _b.actions(createItem, editItem, deleteItems);
        if (!(actions === null || actions === void 0 ? void 0 : actions.length))
            return; // is it needed to set to empty array as well? (ewi)
        // parse to commandbarProps
        let commandBarItems = actions.map(buildMenuItem);
        setMenuItems(commandBarItems);
    }, [props.listProps.menuConfig, props.selectedItems]);
    const buildColumns = useCallback(() => {
        return props.columns.map((_) => {
            var _a, _b, _c;
            let col = _;
            col.onColumnClick = () => props.sortColumn(_);
            const resizable = (_a = props.listProps.columnConfig) === null || _a === void 0 ? void 0 : _a.resizableColumns;
            if (resizable === true || ((_b = resizable === null || resizable === void 0 ? void 0 : resizable.findIndex(_ => _ == col.fieldName)) !== null && _b !== void 0 ? _b : -1) >= 0)
                col.isResizable = true;
            const widthConfig = (_c = props.listProps.columnConfig) === null || _c === void 0 ? void 0 : _c.columnWidths;
            if (widthConfig != null) {
                // do not override width from item schema
                if (typeof widthConfig === 'number' && col.minWidth == null) {
                    col.minWidth = widthConfig;
                }
                else if (Array.isArray(widthConfig)) {
                    const config = widthConfig === null || widthConfig === void 0 ? void 0 : widthConfig.find(_ => _.column == col.fieldName);
                    if ((config === null || config === void 0 ? void 0 : config.width) != null)
                        col.minWidth = config.width;
                }
            }
            return col;
        });
    }, [props.columns, props.sortColumn, (_a = props.listProps.columnConfig) === null || _a === void 0 ? void 0 : _a.resizableColumns]);
    // TODO: move crud operations out to FormList (ewi)
    //       with alot of other stuff as well
    const createItem = useCallback((pre) => __awaiter(void 0, void 0, void 0, function* () {
        let item = {};
        if (pre) {
            let result = yield pre(item);
            if (result === false)
                return false;
        }
        selection.setAllSelected(false);
        props.updateSelectedItems([item]);
        setNewItemMode(true);
        setShowEditor(true);
    }), [props.updateSelectedItems, setNewItemMode, setShowEditor]);
    const editItem = useCallback((pre) => __awaiter(void 0, void 0, void 0, function* () {
        let sItem = Object.assign({}, props.selectedItems[0]);
        if (pre) {
            let result = yield pre(sItem);
            if (result === false)
                return false;
        }
        props.updateSelectedItems([sItem]);
        // setTimeout(() => {
        setNewItemMode(false);
        setShowEditor(true);
        // }, 100); // TODO: why was this needed ? (ewi) (200 before)
    }), [props.updateSelectedItems, setNewItemMode, setShowEditor, props.selectedItems]);
    const deleteItems = useCallback((pre) => __awaiter(void 0, void 0, void 0, function* () {
        const success = yield props.deleteItems(pre);
        if (!success)
            return;
        selection.setAllSelected(false);
    }), [props.deleteItems]);
    const validateForm = useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        var _q;
        let validated = yield ((_q = editorFormRef.current) === null || _q === void 0 ? void 0 : _q.validateItem());
        return validated;
    }), [(_b = editorFormRef.current) === null || _b === void 0 ? void 0 : _b.validateItem]);
    const saveForm = useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        var _r, _s, _t;
        let validated = yield validateForm();
        if (validated == ValidationResult.FailedDontBlock) {
            setShowValidationOverrideConfirm(true);
        }
        else if (validated == ValidationResult.Failed) {
            setValidationFailed(true);
        }
        else if (validated == ValidationResult.Success) {
            setFormLoading(true);
            if ((_s = (_r = props.listProps) === null || _r === void 0 ? void 0 : _r.editorConfig) === null || _s === void 0 ? void 0 : _s.dismissImmediately) {
                new Promise((resolve, reject) => {
                    var _a;
                    props.onItemChange((_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem());
                    resolve();
                });
            }
            else
                yield props.onItemChange((_t = editorFormRef.current) === null || _t === void 0 ? void 0 : _t.getItem());
            setValidationFailed(false);
            setFormLoading(false);
            setShowEditor(false);
        }
    }), [validateForm, setShowValidationOverrideConfirm, setValidationFailed, setFormLoading, (_d = (_c = props.listProps) === null || _c === void 0 ? void 0 : _c.editorConfig) === null || _d === void 0 ? void 0 : _d.dismissImmediately, props.onItemChange, editorFormRef.current, setShowEditor]);
    const dismissForm = useCallback(() => __awaiter(void 0, void 0, void 0, function* () {
        if (newItemMode)
            props.updateSelectedItems([]);
        setNewItemMode(false);
        setShowEditor(false);
    }), [setNewItemMode, setShowEditor, props.updateSelectedItems]);
    const getItem = useCallback(() => {
        var _a;
        return (_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem();
    }, [editorFormRef.current]);
    const onRenderFooterContent = useCallback(() => {
        var _a, _b;
        if (((_b = (_a = props.listProps) === null || _a === void 0 ? void 0 : _a.editorConfig) === null || _b === void 0 ? void 0 : _b.customFooter) != undefined)
            return props.listProps.editorConfig.customFooter(saveForm, dismissForm, validateForm, getItem);
        else
            return (React.createElement(Stack, { horizontal: true },
                formLoading
                    ? React.createElement(Spinner, { styles: { root: { paddingRight: "10px" } } })
                    : React.createElement(PrimaryButton, { text: newItemMode ? lang.texts.areas.common.create : lang.texts.areas.common.save, styles: { root: { marginRight: 8 } }, onClick: saveForm }),
                React.createElement(DefaultButton, { text: lang.texts.areas.common.cancel, onClick: dismissForm, disabled: formLoading }),
                validationFailed && React.createElement("div", { style: { paddingLeft: "10px", paddingTop: "2px" } },
                    React.createElement(Icon, { iconName: "Error", title: "Validation errors", styles: { root: { color: "red", fontSize: "20px", cursor: "help" } } }))));
    }, [validationFailed, formLoading, (_f = (_e = props.listProps) === null || _e === void 0 ? void 0 : _e.editorConfig) === null || _f === void 0 ? void 0 : _f.customFooter, saveForm, dismissForm, validateForm, getItem]);
    const handleColumnReorder = useCallback((draggedIndex, targetIndex) => {
        if (columns == null)
            return;
        const draggedItems = columns[draggedIndex];
        const newColumns = [...columns];
        // insert before the dropped item
        newColumns.splice(draggedIndex, 1);
        newColumns.splice(targetIndex, 0, draggedItems);
        setColumns(newColumns);
    }, [columns, setColumns]);
    return React.createElement(React.Fragment, null,
        (showEditor && ((_g = props.selectedItems) === null || _g === void 0 ? void 0 : _g.length))
            && React.createElement(FluentDialog, { item: props.selectedItems[0], editorConfig: props.listProps.editorConfig, formListProps: props, keyPrefix: (_h = props.keyPrefix) !== null && _h !== void 0 ? _h : "", newItemMode: newItemMode, onRenderFooterContent: onRenderFooterContent, show: (show) => {
                    setValidationFailed(false);
                    setShowEditor(show);
                }, ref: editorFormRef }),
        React.createElement("div", { className: "formbuilder-listcontainer-toolbar" },
            ((_j = props.listProps.searchConfig) === null || _j === void 0 ? void 0 : _j.searchEnabled) &&
                React.createElement(Searcher, { items: items, onSearch: (filteredItems, searchText) => {
                        props.updateFilteredItems(filteredItems);
                        selection.setAllSelected(false);
                        props.updateSelectedItems([]);
                    }, className: "tp-searcher", delay: 600, searchPlaceHolder: props.listProps.searchConfig.searchPlaceHolder, enableTextHighlight: true, searchFields: props.listProps.searchConfig.searchableFields, highLightElements: () => Array.from(document.querySelectorAll('.ms-List .ms-DetailsRow-cell:not(.ms-DetailsRow-cellCheck)'))
                        .filter(_ => {
                        var _a;
                        if (((_a = props.listProps.searchConfig) === null || _a === void 0 ? void 0 : _a.searchableFields) == null)
                            return [];
                        return props.listProps.searchConfig.searchableFields.findIndex((sf) => sf == _.attributes["data-automation-key"].value) >= 0;
                    }), filterOnItemsChange: true }),
            menuItems !== undefined &&
                React.createElement(CommandBar, { items: menuItems, styles: { root: { padding: 0 } } })),
        React.createElement("div", { className: "formbuilder-listcontainer-list" },
            React.createElement(ShimmeredDetailsList, { columns: columns, items: filteredItems, 
                // selectionPreservedOnEmptyClick
                selection: selection, enableShimmer: enableShimmer, shimmerLines: (_l = (_k = props.listProps) === null || _k === void 0 ? void 0 : _k.shimmerConfig) === null || _l === void 0 ? void 0 : _l.shimmerLines, layoutMode: DetailsListLayoutMode.fixedColumns, constrainMode: ConstrainMode.horizontalConstrained, setKey: 'itemId', getKey: (item) => { var _a; return `item-${(_a = props.listProps.config.itemIdentifier(item)) !== null && _a !== void 0 ? _a : id.make()}`; }, onItemInvoked: props.listProps.config.disableItemInvoke ? undefined :
                    (item, index, ev) => {
                        if (item == null) {
                            props.updateSelectedItems([]);
                        }
                        else {
                            props.updateSelectedItems([item]);
                        }
                        editItem();
                    }, columnReorderOptions: (columns === null || columns === void 0 ? void 0 : columns.length) ? { handleColumnReorder } : undefined, useReducedRowRenderer: true })),
        (!enableShimmer && !!!(items === null || items === void 0 ? void 0 : items.length)) && ((_p = (_o = (_m = props.listProps) === null || _m === void 0 ? void 0 : _m.shimmerConfig) === null || _o === void 0 ? void 0 : _o.noItemsElement) !== null && _p !== void 0 ? _p : null),
        React.createElement(Dialog, { hidden: !showValidationOverrideConfirm, title: lang.texts.areas.list.validationRequirementsNotMet.title, onDismiss: () => setShowValidationOverrideConfirm(false), dialogContentProps: {
                subText: lang.texts.areas.list.validationRequirementsNotMet.desctiption
            }, modalProps: {
                isBlocking: true
            } },
            React.createElement(DialogFooter, null,
                React.createElement(PrimaryButton, { text: lang.texts.areas.list.validationRequirementsNotMet.ok, onClick: () => {
                        var _a;
                        setShowValidationOverrideConfirm(false);
                        setShowEditor(false);
                        props.onItemChange((_a = editorFormRef.current) === null || _a === void 0 ? void 0 : _a.getItem());
                    } }),
                React.createElement(DefaultButton, { text: lang.texts.areas.list.validationRequirementsNotMet.cancel, onClick: () => setShowValidationOverrideConfirm(false) }))));
};
export default FluentList;
//# sourceMappingURL=FluentList.js.map